/*
 * simple examples of using condition variables:
 *
 * start some threads but have them wait for main program to signal them
 * before actually beginning work.
 */
#include <cstdio>
#include <cstdlib>
#include <thread>
#include <condition_variable>
#include <string>
#include <vector>

void delay(const int millis);

void thread_body(const std::string id,
        bool& start,
        std::mutex& cv_mutex,
        std::condition_variable& start_threads);

int main(void) {
    std::mutex cv_mutex;
    std::condition_variable start_threads;
    bool start = false;

    printf("starting main\n");
    std::vector<std::thread> threads;
    threads.push_back(std::thread(thread_body, "this", 
                std::ref(start),
                std::ref(cv_mutex),
                std::ref(start_threads)));
    threads.push_back(std::thread(thread_body, "that", 
                std::ref(start),
                std::ref(cv_mutex),
                std::ref(start_threads)));
    printf("in main after creating threads\n");
    delay(4000);
    printf("in main after delay\n");
    {
        std::unique_lock<std::mutex> lock(cv_mutex);
        printf("in main starting threads\n");
        start = true;
        start_threads.notify_all();
    }
    printf("in main after starting threads\n");

    for (auto& th : threads) {
        th.join();
    }
    printf("ending main\n");

    return EXIT_SUCCESS;
}

void delay(const int millis) {
    std::this_thread::sleep_for(std::chrono::milliseconds(millis));
}

void thread_body(const std::string id, 
        bool& start,
        std::mutex& cv_mutex,
        std::condition_variable& start_threads)
{
    printf("initializing thread %s\n", id.c_str());
    {
        std::unique_lock<std::mutex> lock(cv_mutex);
        // while (!start) {
        start_threads.wait(lock, [&]{ return start; });
        // }
    }
    printf("starting thread %s\n", id.c_str());
    delay(2000);
    printf("ending thread %s\n", id.c_str());
}
